/******************************************************************************/
/*  Files to Include                                                          */
/******************************************************************************/
#ifdef __XC32
    #include <xc.h>          /* Defines special funciton registers, CP0 regs  */
#endif

#include <plib.h>           /* Include to use PIC32 peripheral libraries      */
#include "system.h"         /* System funct/params, like osc/periph config    */

/******************************************************************************/
/* Global Variable Declaration                                                */
/******************************************************************************/
#define FPB_FREQUENCY   4000000
#define OUTPUT  0
#define INPUT   1

#define DE_TRIS TRISGbits.TRISG6
#define DE_LAT  LATGbits.LATG6
#define DE_ON   DE_LAT = 1
#define DE_OFF  DE_LAT = 0
#define RE_TRIS TRISFbits.TRISF0
#define RE_LAT  LATFbits.LATF0
#define RE_ON   RE_LAT = 0
#define RE_OFF  RE_LAT = 1

void UART1_Init (unsigned long int BaudRate)
{
    U1MODEbits.BRGH = 0;
    U1BRG = ((FPB_FREQUENCY/BaudRate)/16)-1;
    U1STAbits.UTXEN = 1;
    U1STAbits.URXEN = 1;
    U1MODEbits.ON = 1;
}

void UART1_Transmit (unsigned char Data)
{
    while (U1STAbits.UTXBF);
    U1TXREG = Data;
}

unsigned char UART1_Receive (void)
{
    if (U1STAbits.URXDA)
        return U1RXREG;
    else
        return 0;
}

void UART2_Init (unsigned long int BaudRate)
{
    U2MODEbits.BRGH = 0;
    U2BRG = ((FPB_FREQUENCY/BaudRate)/16)-1;
    U2STAbits.UTXEN = 1;
    U2STAbits.URXEN = 1;
    U2MODEbits.ON = 1;
}

void UART2_Transmit (unsigned char Data)
{
    while (U2STAbits.UTXBF);
    U2TXREG = Data;
}

unsigned char UART2_Receive (void)
{
    if (U2STAbits.URXDA)
        return U2RXREG;
    else
        return 0;
}

void Delay (volatile unsigned long int Time)
{
    while (Time--);
}

int main (void)
{
    char c1, c2;
    int i=0;

    SYSTEMConfig (SYS_FREQ, SYS_CFG_ALL);
    INTConfigureSystem (INT_SYSTEM_CONFIG_MULT_VECTOR);

    RE_TRIS = OUTPUT;
    DE_TRIS = OUTPUT;
    RE_ON;
    DE_OFF;

    UART1_Init (9600);
    UART2_Init (9600);

    while (1)
    {
        c1 = UART1_Receive ();
        if (c1)
        {
            DE_ON;
            UART2_Transmit (c1);
            for (i=1000; i; i--);
            DE_OFF;
        }

        c2 = UART2_Receive ();
        if (c2)
            UART1_Transmit (c2);
    }
}
